# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from pymongo import MongoClient


class MongoHelper(object):

    def __init__(self):
        is_container = True if os.environ.get('TEST_CONTAINER') else False
        self.mongo = MongoClient(host="database" if is_container else "127.0.0.1")

    def drop_database(self):
        self.mongo.drop_database('almanach')
